package org.fuseri.moduleexercise.answer;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import jakarta.persistence.EntityNotFoundException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import org.fuseri.model.dto.exercise.AnswerCreateDto;
import org.fuseri.model.dto.exercise.AnswerDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

/**
 * Represent a REST API controller for answers
 * Handle HTTP requests related to answers
 */
@RestController
@RequestMapping("/answers")
public class AnswerController {

    private final AnswerFacade facade;

    @Autowired
    public AnswerController(AnswerFacade facade) {
        this.facade = facade;
    }

    /**
     * Update an answer
     *
     * @param id  of answer to update
     * @param dto dto with updated answer information
     * @return A ResponseEntity with an AnswerDto object representing the updated answer on an HTTP status code of 200 if the update was successful.
     * or a NOT_FOUND response if the answer ID is invalid
     */
    @Operation(summary = "Update an answer", description = "Updates an answer with the specified ID.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Answer with the specified ID updated successfully."),
            @ApiResponse(responseCode = "400", description = "Invalid input."),
            @ApiResponse(responseCode = "404", description = "Answer with the specified ID was not found.")
    })
    @PutMapping("/{id}")
    public ResponseEntity<AnswerDto> update(@NotNull @PathVariable long id, @Valid @RequestBody AnswerCreateDto dto) {
        try {
            return ResponseEntity.ok(facade.update(id, dto));
        } catch (EntityNotFoundException e) {
            return ResponseEntity.notFound().build();
        }
    }

    /**
     * Delete answer with the specified ID
     *
     * @param id of answer to delete
     * @throws ResponseStatusException if answer with specified id does not exist
     */
    @Operation(summary = "Delete an answer with specified ID", description = "Deletes an answer with the specified ID.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "204", description = "Answer with the specified ID deleted successfully."),
            @ApiResponse(responseCode = "404", description = "Answer with the specified ID was not found.")
    })
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> delete(@NotNull @PathVariable long id) {
        try {
            facade.delete(id);
            return ResponseEntity.noContent().build();
        } catch (EntityNotFoundException e) {
            return ResponseEntity.notFound().build();
        }
    }
}
